@@ -15,13 +15,12 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from locale import getpreferredencoding
+import logging
+import os.path
 from parsley import makeGrammar
 import subprocess
-
-if not getattr(subprocess, 'check_output', None):
-    import bindep.support_py26
-    # shut pyflakes up.
-    bindep.support_py26
+import sys
 
 
 debversion_grammar = """
@@ -45,10 +44,12 @@ rule = <name>:name selector?:selector version?:version ('\n'|comment) -> (
     name, selector or [], version or [])
 lowercase = ('a'|'b'|'c'|'d'|'e'|'f'|'g'|'h'|'i'|'j'|'k'|'l'|'m'|'n'|'o'|'p'
             |'q'|'r'|'s'|'t'|'u'|'v'|'w'|'x'|'y'|'z')
-name = letterOrDigit:start (letterOrDigit|'.'|'+'|'-'|'/')+:rest
+name = letterOrDigit:start (letterOrDigit|'.'|'+'|'-'|'_'|'/')+:rest
 ws = ' '+
-profile = ('!'?:neg <(lowercase|digit|':'|'-')+>:name) -> (neg!='!', name)
-selector = ws '[' profile:p1 (ws profile)*:p2 ']' -> [p1] + p2
+profile = ('!'?:neg <(lowercase|digit|':'|'-'|'.')+>:name) -> (neg!='!', name)
+profiles = '(' (ws? profile)*:p ws? ')' -> p
+group = profiles | profile
+selector = ws '[' (ws? group)*:p ws? ']' -> p
 oneversion = <('<=' | '<' | '!=' | '==' | '>=' | '>')>:rel <debversion>:v -> (
     rel, v)
 version = ws oneversion:v1 (',' oneversion)*:v2 -> [v1] + v2
@@ -58,6 +59,50 @@ blank = ws? '\n' -> None
 """
 
 
+def get_depends(filename=None):
+    fd = get_depends_file(filename)
+    if not fd:
+        return None
+    return Depends(fd.read())
+
+
+def get_depends_file(filename=None):
+    log = logging.getLogger(__name__)
+    if filename == "-":
+        return sys.stdin
+    elif filename:
+        try:
+            fd = open(filename, 'rt')
+        except IOError:
+            log.error('Error reading file %s.' % filename)
+            return None
+    else:
+        if (os.path.isfile('bindep.txt') and
+            os.path.isfile('other-requirements.txt')):
+            log.error(
+                'Both bindep.txt and other-requirements.txt '
+                'files exist, choose one.')
+            return None
+        if os.path.isfile('bindep.txt'):
+            try:
+                fd = open('bindep.txt', 'rt')
+            except IOError:
+                log.error('Error reading file bindep.txt.')
+                return None
+        elif os.path.isfile('other-requirements.txt'):
+            try:
+                fd = open('other-requirements.txt', 'rt')
+            except IOError:
+                log.error('Error reading file other-requirements.txt.')
+                return None
+        else:
+            log.error(
+                'Neither file bindep.txt nor file '
+                'other-requirements.txt exist.')
+            return None
+    return fd
+
+
 class Depends(object):
     """Project dependencies."""
 
@@ -95,7 +140,11 @@ class Depends(object):
         """
         platform = []
         user = []
-        for sense, profile in rule[1]:
+        for group in rule[1]:
+            if isinstance(group, list):
+                user.append(group)
+                continue
+            sense, profile = group
             if profile.startswith("platform:"):
                 platform.append((sense, profile))
             else:
@@ -116,8 +165,16 @@ class Depends(object):
         # selectors we need a match.
         positive = False
         match_found = False
+        group_found = False
+        group_match_found = False
         negative = False
-        for sense, profile in partition_rule:
+        for group in partition_rule:
+            if isinstance(group, list):
+                group_found = True
+                if self._match_all(group, profiles):
+                    group_match_found = True
+                continue
+            sense, profile = group
             if sense:
                 positive = True
                 if profile in profiles:
@@ -126,10 +183,26 @@ class Depends(object):
                 if profile in profiles:
                     negative = True
                     break
-        if not negative and (match_found or not positive):
+        if not negative:
+            if group_match_found or match_found:
+                return True
+            if not group_found and not positive:
                 return True
         return False
 
+    def _match_all(self, partition_rules, profiles):
+        """Evaluate rules. Do they all match the profiles?
+
+        :return Result True if all profiles match else False
+        """
+        def matches(sense, profile, profiles):
+            return sense if profile in profiles else not sense
+
+        for sense, profile in partition_rules:
+            if not matches(sense, profile, profiles):
+                return False
+        return True
+
     def active_rules(self, profiles):
         """Return the rules active given profiles.
 
@@ -151,6 +224,25 @@ class Depends(object):
                 result.append(rule)
         return result
 
+    def list_all_packages(self, rules, output_format='newline'):
+        """Print a list of all packages that are required on this platform
+        according to the passed in rules. This is useful if we want to build
+        RPMs based on the deps listed in bindeps.txt
+
+        :param rules: A list of rules, as returned by active_rules.
+        :param output_format: The format to print the output in. Currently
+        we support newline format which will print 1 package per line, and
+        csv format which prints a csv list.
+        :return: List of all required packages regardless of whether they are
+        missing.
+        """
+        packages_list = [rule[0] for rule in rules]
+        if output_format == 'csv':
+            logging.info(','.join(packages_list))
+        elif output_format == 'newline':
+            logging.info('\n'.join(packages_list))
+        return packages_list
+
     def check_rules(self, rules):
         """Evaluate rules against the local environment.
 
@@ -163,6 +255,7 @@ class Depends(object):
             installed = self.platform.get_pkg_version(rule[0])
             if not installed:
                 missing.add(rule[0])
+                continue
             for operator, constraint in rule[2]:
                 if not _eval(installed, operator, constraint):
                     incompatible.append(
@@ -182,9 +275,14 @@ class Depends(object):
         return sorted(profiles)
 
     def platform_profiles(self):
-        output = subprocess.check_output(
-            ["lsb_release", "-cirs"],
-            stderr=subprocess.STDOUT).decode('utf-8')
+        try:
+            output = subprocess.check_output(
+                ["lsb_release", "-cirs"],
+                stderr=subprocess.STDOUT).decode(getpreferredencoding(False))
+        except OSError:
+            log = logging.getLogger(__name__)
+            log.error('Unable to execute lsb_release. Is it installed?')
+            raise
         lsbinfo = output.lower().split()
         # NOTE(toabctl): distro can be more than one string (i.e. "SUSE LINUX")
         codename = lsbinfo[len(lsbinfo) - 1:len(lsbinfo)][0]
@@ -199,7 +297,21 @@ class Depends(object):
         if distro in ["debian", "ubuntu"]:
             atoms.add("dpkg")
             self.platform = Dpkg()
-        elif distro in ["centos", "fedora", "opensuse", "suselinux"]:
+        elif distro in ["amazonami", "centos", "redhatenterpriseserver",
+                        "fedora", "opensuseproject", "opensusetumbleweed",
+                        "suselinux"]:
+            if distro == "redhatenterpriseserver":
+                # just short alias
+                atoms.add("rhel")
+            elif distro in ["opensuseproject", "opensusetumbleweed"]:
+                # just short alias
+                atoms.add("opensuse")
+            # Family aliases
+            if 'suse' in distro:
+                atoms.add("suse")
+            else:
+                atoms.add("redhat")
+
             atoms.add("rpm")
             self.platform = Rpm()
         elif distro in ["gentoo"]:
@@ -232,11 +344,13 @@ class Dpkg(Platform):
         try:
             output = subprocess.check_output(
                 ["dpkg-query", "-W", "-f", "${Package} ${Status} ${Version}\n",
-                 pkg_name], stderr=subprocess.STDOUT).decode('utf-8')
+                 pkg_name],
+                stderr=subprocess.STDOUT).decode(getpreferredencoding(False))
         except subprocess.CalledProcessError as e:
+            eoutput = e.output.decode(getpreferredencoding(False))
             if (e.returncode == 1 and
-                (e.output.startswith('dpkg-query: no packages found') or
-                 e.output.startswith('No packages found matching'))):
+                (eoutput.startswith('dpkg-query: no packages found') or
+                 eoutput.startswith('No packages found matching'))):
                 return None
             raise
         # output looks like
@@ -259,11 +373,14 @@ class Rpm(Platform):
         try:
             output = subprocess.check_output(
                 ["rpm", "--qf",
-                 "%{NAME} %|EPOCH?{%{EPOCH}:}|%{VERSION}-%{RELEASE}\n", "-q",
-                 pkg_name], stderr=subprocess.STDOUT).decode('utf-8')
+                 "%{NAME} %|EPOCH?{%{EPOCH}:}|%{VERSION}-%{RELEASE}\n",
+                 "--whatprovides", "-q", pkg_name],
+                stderr=subprocess.STDOUT).decode(getpreferredencoding(False))
         except subprocess.CalledProcessError as e:
+            eoutput = e.output.decode(getpreferredencoding(False))
             if (e.returncode == 1 and
-                e.output.strip().endswith('is not installed')):
+                (eoutput.strip().endswith('is not installed') or
+                 (eoutput.strip().startswith('no package provides')))):
                 return None
             raise
         # output looks like
@@ -285,7 +402,7 @@ class Emerge(Platform):
         try:
             output = subprocess.check_output(
                 ['equery', 'l', '--format=\'$version\'', pkg_name],
-                stderr=subprocess.STDOUT).decode('utf-8')
+                stderr=subprocess.STDOUT).decode(getpreferredencoding(False))
         except subprocess.CalledProcessError as e:
             if e.returncode == 3:
                 return None
@@ -307,9 +424,10 @@ class Pacman(Platform):
         try:
             output = subprocess.check_output(
                 ['pacman', '-Q', pkg_name],
-                stderr=subprocess.STDOUT)
+                stderr=subprocess.STDOUT).decode(getpreferredencoding(False))
         except subprocess.CalledProcessError as e:
-            if e.returncode == 1 and e.output.endswith('was not found'):
+            eoutput = e.output.decode(getpreferredencoding(False))
+            if e.returncode == 1 and eoutput.endswith('was not found'):
                 return None
             raise
         # output looks like
