@@ -47,8 +47,9 @@ class TestDepends(TestCase):
         self.assertEqual([], depends.profiles())
 
     def test_platform_profiles_succeeds(self):
-        depends = Depends("")
-        self.assertIsInstance(depends.platform_profiles(), list)
+        with self._mock_lsb('Ubuntu'):
+            depends = Depends("")
+            self.assertIsInstance(depends.platform_profiles(), list)
 
     @contextlib.contextmanager
     def _mock_lsb(self, platform):
@@ -62,29 +63,70 @@ class TestDepends(TestCase):
         mock_checkoutput.assert_called_once_with(["lsb_release", "-cirs"],
                                                  stderr=subprocess.STDOUT)
 
+    def test_detects_amazon_linux(self):
+        with self._mock_lsb("AmazonAMI"):
+            depends = Depends("")
+            self.assertThat(
+                depends.platform_profiles(), Contains("platform:amazonami"))
+
     def test_detects_centos(self):
         with self._mock_lsb("CentOS"):
             depends = Depends("")
+            platform_profiles = depends.platform_profiles()
+            self.assertThat(platform_profiles, Contains("platform:centos"))
+            self.assertThat(platform_profiles, Contains("platform:redhat"))
+
+    def test_detects_rhel(self):
+        with self._mock_lsb("RedHatEnterpriseServer"):
+            depends = Depends("")
+            platform_profiles = depends.platform_profiles()
             self.assertThat(
-                depends.platform_profiles(), Contains("platform:centos"))
+                platform_profiles,
+                Contains("platform:redhatenterpriseserver"))
+            self.assertThat(
+                platform_profiles,
+                Contains("platform:rhel"))
+            self.assertThat(
+                platform_profiles,
+                Contains("platform:redhat"))
 
     def test_detects_fedora(self):
         with self._mock_lsb("Fedora"):
             depends = Depends("")
-            self.assertThat(
-                depends.platform_profiles(), Contains("platform:fedora"))
+            platform_profiles = depends.platform_profiles()
+            self.assertThat(platform_profiles, Contains("platform:fedora"))
+            self.assertThat(platform_profiles, Contains("platform:redhat"))
 
-    def test_detects_opensuse(self):
-        with self._mock_lsb("openSUSE"):
+    def test_detects_opensuse_project(self):
+        with self._mock_lsb("openSUSE Project"):
             depends = Depends("")
-            self.assertThat(
-                depends.platform_profiles(), Contains("platform:opensuse"))
+            platform_profiles = depends.platform_profiles()
+            self.assertThat(platform_profiles,
+                            Contains("platform:opensuseproject"))
+            self.assertThat(platform_profiles,
+                            Contains("platform:opensuse"))
+            self.assertThat(platform_profiles,
+                            Contains("platform:opensuseproject-14.04"))
+            self.assertThat(platform_profiles,
+                            Contains("platform:suse"))
+
+    def test_detects_opensuse_tumbleweed(self):
+        with self._mock_lsb("openSUSE Tumbleweed"):
+            depends = Depends("")
+            platform_profiles = depends.platform_profiles()
+            self.assertThat(platform_profiles,
+                            Contains("platform:opensusetumbleweed"))
+            self.assertThat(platform_profiles,
+                            Contains("platform:opensuse"))
+            self.assertThat(platform_profiles,
+                            Contains("platform:suse"))
 
     def test_detects_suse_linux(self):
         with self._mock_lsb("SUSE Linux"):
             depends = Depends("")
-            self.assertThat(
-                depends.platform_profiles(), Contains("platform:suselinux"))
+            platform_profiles = depends.platform_profiles()
+            self.assertThat(platform_profiles, Contains("platform:suselinux"))
+            self.assertThat(platform_profiles, Contains("platform:suse"))
 
     def test_detects_ubuntu(self):
         with self._mock_lsb("Ubuntu"):
@@ -118,6 +160,13 @@ class TestDepends(TestCase):
                 depends.platform_profiles(), Contains("platform:rpm"))
             self.assertIsInstance(depends.platform, Rpm)
 
+    def test_rhel_implies_rpm(self):
+        with self._mock_lsb("RedHatEnterpriseServer"):
+            depends = Depends("")
+            self.assertThat(
+                depends.platform_profiles(), Contains("platform:rpm"))
+            self.assertIsInstance(depends.platform, Rpm)
+
     def test_fedora_implies_rpm(self):
         with self._mock_lsb("Fedora"):
             depends = Depends("")
@@ -125,8 +174,15 @@ class TestDepends(TestCase):
                 depends.platform_profiles(), Contains("platform:rpm"))
             self.assertIsInstance(depends.platform, Rpm)
 
-    def test_opensuse_implies_rpm(self):
-        with self._mock_lsb("openSUSE"):
+    def test_opensuse_project_implies_rpm(self):
+        with self._mock_lsb("openSUSE Project"):
+            depends = Depends("")
+            self.assertThat(
+                depends.platform_profiles(), Contains("platform:rpm"))
+            self.assertIsInstance(depends.platform, Rpm)
+
+    def test_opensuse_tumbleweed_implies_rpm(self):
+        with self._mock_lsb("openSUSE Tumbleweed"):
             depends = Depends("")
             self.assertThat(
                 depends.platform_profiles(), Contains("platform:rpm"))
@@ -153,6 +209,12 @@ class TestDepends(TestCase):
                 depends.platform_profiles(), Contains("platform:pacman"))
             self.assertIsInstance(depends.platform, Pacman)
 
+    def test_missing_lsb_release(self):
+        with mock.patch('subprocess.check_output') as mock_co:
+            mock_co.side_effect = OSError
+            depends = Depends("")
+            self.assertRaises(OSError, depends.platform_profiles)
+
     def test_finds_profiles(self):
         depends = Depends(dedent("""\
             foo
@@ -174,6 +236,47 @@ class TestDepends(TestCase):
             [("foo", [(False, "bar"), (True, "baz"), (True, "quux")], [])],
             depends._rules)
 
+    def test_single_group_only(self):
+        depends = Depends("foo [(bar)]\n")
+        self.assertTrue(depends._evaluate(depends._rules[0][1], ["bar"]))
+        self.assertFalse(depends._evaluate(depends._rules[0][1], ["baz"]))
+
+    def test_multiple_groups_only(self):
+        depends = Depends("foo [(bar baz) (quux)]\n")
+        self.assertTrue(depends._evaluate(depends._rules[0][1],
+                                          ["bar", "baz"]))
+        self.assertTrue(depends._evaluate(depends._rules[0][1], ["quux"]))
+        self.assertFalse(depends._evaluate(depends._rules[0][1], ["baz"]))
+        self.assertFalse(depends._evaluate(depends._rules[0][1], ["bar"]))
+
+    def test_whitespace(self):
+        depends = Depends("foo [ ( bar !baz ) quux ]\n")
+        self.assertEqual(
+            [("foo", [[(True, "bar"), (False, "baz")], (True, "quux")], [])],
+            depends._rules)
+
+    def test_group_selectors(self):
+        depends = Depends("foo [(bar !baz) quux]\n")
+        self.assertEqual(
+            [("foo", [[(True, "bar"), (False, "baz")], (True, "quux")], [])],
+            depends._rules)
+
+    def test_multiple_group_selectors(self):
+        depends = Depends("foo [(bar baz) (baz quux)]\n")
+        parsed_profiles = [
+            [(True, "bar"), (True, "baz")],
+            [(True, "baz"), (True, "quux")],
+        ]
+        self.assertEqual(
+            [("foo", parsed_profiles, [])],
+            depends._rules)
+
+    def test_single_profile_group_selectors(self):
+        depends = Depends("foo [(bar) (!baz)]\n")
+        self.assertEqual(
+            [("foo", [[(True, "bar")], [(False, "baz")]], [])],
+            depends._rules)
+
     def test_versions(self):
         depends = Depends("foo <=1,!=2\n")
         self.assertEqual(
@@ -208,6 +311,17 @@ class TestDepends(TestCase):
             [('missing', ['foo'])], depends.check_rules([("foo", [], [])]))
         mock_depend_platform.assert_called_once_with("foo")
 
+    def test_check_rule_missing_version(self):
+        depends = Depends("")
+        depends.platform = mock.MagicMock()
+        mock_depend_platform = self.useFixture(
+            fixtures.MockPatchObject(depends.platform, 'get_pkg_version',
+                                     return_value=None)).mock
+        self.assertEqual(
+            [('missing', ['foo'])],
+            depends.check_rules([("foo", [], [(">=", "1.2.3")])]))
+        mock_depend_platform.assert_called_once_with("foo")
+
     def test_check_rule_present(self):
         depends = Depends("")
         depends.platform = mock.MagicMock()
@@ -235,6 +349,7 @@ class TestDepends(TestCase):
             bar [something]
             category/packagename # for gentoo
             baz [platform:this platform:that-those]
+            blaz [platform:rpm !platform:opensuseproject-42.2]
             quux [anotherthing !nothing] <=12
             womp # and a comment
             # a standalone comment and a blank line
@@ -276,6 +391,65 @@ class TestDepends(TestCase):
             set(r[0] for r in depends.active_rules(['platform:rpm', 'test'])),
             Equals({"install5", "install6", "install8", "install9"}))
 
+    def test_list_all(self):
+        depends = Depends(dedent("""\
+            install1
+            install2 [test]
+            install3 [platform:rpm]
+            install4 [platform:dpkg]
+            install5 [quark]
+            install6 [platform:dpkg test]
+            install7 [quark test]
+            install8 [platform:dpkg platform:rpm]
+            install9 [platform:dpkg platform:rpm test]
+            installA [!platform:dpkg]
+            installB [!platform:dpkg test]
+            installC [!platform:dpkg !test]
+            installD [platform:dpkg !test]
+            installE [platform:dpkg !platform:rpm]
+            installF [platform:dpkg !platform:rpm test]
+            installG [!platform:dpkg !platform:rpm]
+            installH [!platform:dpkg !platform:rpm test]
+            installI [!platform:dpkg !platform:rpm !test]
+            installJ [platform:dpkg !platform:rpm !test]
+            """))
+
+        rules_dpkg = depends.active_rules(['platform:dpkg'])
+        result_dpkg = set(r[0] for r in rules_dpkg)
+        self.assertEqual(result_dpkg,
+                         set(depends.list_all_packages(rules_dpkg,
+                             output_format='newline')))
+        self.assertEqual(result_dpkg,
+                         set(depends.list_all_packages(rules_dpkg,
+                             output_format='csv')))
+
+        rules_dpkg_test = depends.active_rules(['platform:dpkg', 'test'])
+        result_dpkg_test = set(r[0] for r in rules_dpkg_test)
+        self.assertEqual(result_dpkg_test, set(depends.list_all_packages(
+                                               rules_dpkg_test,
+                                               output_format='newline')))
+        self.assertEqual(result_dpkg_test, set(depends.list_all_packages(
+                                               rules_dpkg_test,
+                                               output_format='csv')))
+
+        rules_rpm = depends.active_rules(['platform:rpm'])
+        result_rpm = set(r[0] for r in rules_rpm)
+        self.assertEqual(result_rpm, set(depends.list_all_packages(rules_rpm,
+                                         output_format='newline')))
+        self.assertEqual(result_rpm, set(depends.list_all_packages(rules_rpm,
+                                         output_format='csv')))
+
+        rules_rpm_test = depends.active_rules(['platform:rpm', 'test'])
+        result_rpm_test = set(r[0] for r in rules_rpm_test)
+        self.assertEqual(result_rpm_test,
+                         set(depends.list_all_packages(
+                             rules_rpm_test,
+                             output_format='newline')))
+        self.assertEqual(result_rpm_test,
+                         set(depends.list_all_packages(
+                             rules_rpm_test,
+                             output_format='csv')))
+
     def test_platforms(self):
         depends = Depends(dedent("""\
             install1
@@ -352,7 +526,7 @@ class TestDpkg(TestCase):
 
         def _side_effect_raise(*args, **kwargs):
             raise subprocess.CalledProcessError(
-                1, [], "dpkg-query: no packages found matching foo\n")
+                1, [], b"dpkg-query: no packages found matching foo\n")
 
         mock_checkoutput.side_effect = _side_effect_raise
         self.assertEqual(None, platform.get_pkg_version("foo"))
@@ -426,7 +600,7 @@ class TestPacman(TestCase):
 
         def _side_effect_raise(*args, **kwargs):
             raise subprocess.CalledProcessError(
-                1, [], "error: package 'foo' was not found")
+                1, [], b"error: package 'foo' was not found")
 
         mock_checkoutput = self.useFixture(
             fixtures.MockPatchObject(subprocess, "check_output")).mock
@@ -442,7 +616,7 @@ class TestPacman(TestCase):
         platform = Pacman()
         mock_checkoutput = self.useFixture(
             fixtures.MockPatchObject(subprocess, "check_output")).mock
-        mock_checkoutput.return_value = 'foo 4.0.0-2'
+        mock_checkoutput.return_value = b'foo 4.0.0-2'
         self.assertEqual("4.0.0-2", platform.get_pkg_version("foo"))
         mock_checkoutput.assert_called_once_with(
             ['pacman', '-Q', 'foo'],
@@ -459,7 +633,7 @@ class TestRpm(TestCase):
 
         def _side_effect_raise(*args, **kwargs):
             raise subprocess.CalledProcessError(
-                1, [], "package foo is not installed\n")
+                1, [], b"package foo is not installed\n")
 
         mock_checkoutput = self.useFixture(
             fixtures.MockPatchObject(subprocess, 'check_output')).mock
@@ -467,8 +641,8 @@ class TestRpm(TestCase):
         self.assertEqual(None, platform.get_pkg_version("foo"))
         mock_checkoutput.assert_called_once_with(
             ["rpm", "--qf",
-             "%{NAME} %|EPOCH?{%{EPOCH}:}|%{VERSION}-%{RELEASE}\n", "-q",
-             "foo"],
+             "%{NAME} %|EPOCH?{%{EPOCH}:}|%{VERSION}-%{RELEASE}\n",
+             "--whatprovides", "-q", "foo"],
             stderr=subprocess.STDOUT)
         self.assertEqual(None, platform.get_pkg_version("foo"))
 
@@ -480,8 +654,8 @@ class TestRpm(TestCase):
         self.assertEqual("4.0.0-0.el6", platform.get_pkg_version("foo"))
         mock_checkoutput.assert_called_once_with(
             ["rpm", "--qf",
-             "%{NAME} %|EPOCH?{%{EPOCH}:}|%{VERSION}-%{RELEASE}\n", "-q",
-             "foo"],
+             "%{NAME} %|EPOCH?{%{EPOCH}:}|%{VERSION}-%{RELEASE}\n",
+             "--whatprovides", "-q", "foo"],
             stderr=subprocess.STDOUT)
 
 
